/** Gradient descent
 *  Numerical Algorithms (Prof. M. Rumpf) WS 2012/13
 *
 *  \file   gradientDescent.h
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   15.01.2013
 *
 */


#ifndef __GRADIENTDESCENT_H
#define __GRADIENTDESCENT_H

#include "../vecmat/operator.h"
#include "../fe/boundaryMasks.h"

//! \brief Grund fuer Abbruch
//! - TOL_Reached: Gewuenschte Genauigkeit erreicht
//! - MaxIterReached: Maximal zulaessige Anzahl von Schritten (_maxIter) ueberschritten
//! - NoStepsizeFound: Minimale Schrittweite (_minStepsize) unterschritten
typedef enum { TOL_Reached, ETOL_Reached, MaxIterReached, NoStepsizeFound, NotTerminated } TerminationReason;

//! Gradientenabstiegs-Klasse
class GradientDescent
{
    public:
        GradientDescent(const Operator<Vector,double> & energy, const Operator<Vector,Vector> & gradient);

        void apply(const Vector & x_0, Vector & x_final, double lambda = 1.);

        void init();
        TerminationReason performSingleStep(Vector & x);
        void checkTermination() const;

        void setVerbose(bool verbose = true);
        void setQuadraticApproximation(bool quad = true);

    protected:
        double findStepsizeWithArmijo(const Vector & x, const Vector & p, const Vector & grad);

        void printHeader() const;
        void printIterate( double gradNorm, double energy ) const;

        const Operator<Vector,double> & _energy;
        const Operator<Vector,Vector> & _gradient;

        double _TOL;
        double _ETOL;
        int    _currentIter;
        int    _maxIter;
        double _armijoBeta;
        double _minStepsize;
        double _maxStepsize;

        bool   _verbose;
        bool   _useQuadraticApproximation;

        TerminationReason _lastTerminationReason;
        double _lastStepsize;
        double _energyOld;
        double _energyNew;
        double _evaluations;
};

#endif
