/** Elastic problem
 *
 *  \file   ElasticProblem.h
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   21.01.2013
 *
 */


#ifndef __ELASTICPROBLEM_H
#define __ELASTICPROBLEM_H

#include "LinearFEOperator.h"
#include "elastBilf.h"
#include "boundaryMasks.h"
#include "../vecmat/solver.h"


//! different loading scenarios for shape optimization
typedef enum { SQUARE, CANTILEVER, XMAS } ScenarioType;

//! class for setting up and solving a elasticity problem with phase fields
class ElasticProblem
{
    public:
        explicit ElasticProblem( const double eps, ScenarioType scenario );
        void updatePhasefield( const Vector & pf );
        const Vector & getSolution() const;
        const Vector & getPhasefield() const;
        const Grid   & getGrid() const;

    protected:
        void solve();
        void generateCircularPhasefield ( Vector & pf, const Grid & grid, const SmallVector2<> & mid, const double radius, const double eps );
        void generateBarPhasefield ( Vector & pf, const Grid & grid, const double width, const double y, const double eps );
        void generateMysteriousPhasefield( Vector & pf, const Grid & grid, const double eps );
        void generateMysteriousGrid( Grid & grid );

        Grid                  _grid;
        int                   _N;
        Vector                _pf;
        Vector                _rhs;
        Vector                _u;
        SmallVector2<>        _load;
        DirichletBoundaryMask _bdryMask;
};

#endif
