/** Linearer Finite-Elemente-Operator fuer das Programmierpraktikum
 *
 *  \file   LinearFEOperator.h
 *  \author Stefan W. von Deylen, Benedict Geihe
 *  \date   01.10.2008
 *
 */

#ifndef __LINEARFEOPERATOR_H
#define __LINEARFEOPERATOR_H

#include "../triang/grid.h"
#include "../vecmat/vector.h"
#include "../vecmat/matrix.h"
#include "../vecmat/sparse.h"
#include "../vecmat/tensor.h"
#include "../fe/boundaryMasks.h"


//! Klasse fuer linearen Finite-Elemente-Operator.
//! Abgeleitet von SparseMatrix.
//! Template-Argumente fuer Bilinearform und Quadratur.
template <typename BilfType, typename QuadType, int Dim = 1>
class LinearFEOperator : public SparseMatrix
{
    public:
        LinearFEOperator( const Grid & grid, BilfType & bilf );

        void apply   ( const Vector& x, Vector& b, double lambda = 1.0 ) const;
        void applyAdd( const Vector& x, Vector& b, double lambda = 1.0 ) const;

    protected:
        void assembleLocalMatrix( const Triangle & triangle, Matrix & mat, int k, int l ) const;

        const Grid & _grid;
        const BilfType & _bilf;
};


//! Crazy workaround: As we cannot partially specialize member function assembleLocalMatrix we introduce
//!                   another class that just calls operator() of the bilinear and which can be partially specialized
template <typename BilfType, int Dim>
struct BilfEvaluator
{
    public:
        static double evaluate( const BilfType & bilf, const Triangle & t, const BarCoord & q, int i, int j, int k, int l )
        {
            return bilf( t, q, i, j, k, l);
        }
};

template <typename BilfType>
struct BilfEvaluator< BilfType, 1>
{
    public:
        static double evaluate (const BilfType & bilf, const Triangle & t, const BarCoord & q, int i, int j, int /*k*/, int /*l*/ )
        {
            return bilf( t, q, i, j);
        }
};

#endif
