/** Nichtlinearer Finite-Elemente-Operator fuer das Programmierpraktikum
 *
 *  \file   NonlinearFEOperator.cpp
 *  \author Stefan W. von Deylen, Benedict Geihe
 *  \date   21.11.2008
 *
 */


#include "NonlinearFEOperator.h"
#include "PhasefieldOps.h"

//! Konstruktor ohne Integrand
template <typename RangeType, typename QuadType, typename IntegrandType>
NonlinearFEOperator<RangeType,QuadType,IntegrandType>::NonlinearFEOperator( const Grid & grid )
    : _grid ( grid )
    , _integrand( new IntegrandType )
    , _deleteIntegrandOnDestruction ( true )
{ }

//! Konstruktor mit Integrand
template <typename RangeType, typename QuadType, typename IntegrandType>
NonlinearFEOperator<RangeType,QuadType,IntegrandType>::NonlinearFEOperator( const Grid & grid, const IntegrandType & integrand )
    : _grid ( grid )
    , _integrand( &integrand )
    , _deleteIntegrandOnDestruction ( false )
{ }

//! Destruktor
template <typename RangeType, typename QuadType, typename IntegrandType>
NonlinearFEOperator<RangeType,QuadType,IntegrandType>::~NonlinearFEOperator()
{
    if (_deleteIntegrandOnDestruction)
        delete _integrand;
}

//! Implementierung der applyAdd-Funktion
template <typename RangeType, typename QuadType, typename IntegrandType>
void NonlinearFEOperator<RangeType,QuadType,IntegrandType>::applyAdd( const Vector& x, RangeType& b, double lambda ) const
{
    for ( typename Grid::TriangleConstIterator iter = this->_grid; iter.notAtEnd(); ++iter )
    {
        double area = iter->getVolume();
        for ( int k =0; k < QuadType::_numPoints; ++k )
        {
            _integrand->addEval( x, b, *iter, QuadType::_barCoord[k], QuadType::_weight[k] * area * lambda );
        }
    }
}

// Explizite Instanzierungen
template class NonlinearFEOperator<double, BarQuad1, PhasefieldNonLin>;
template class NonlinearFEOperator<double, BarQuad2, PhasefieldNonLin>;
template class NonlinearFEOperator<double, BarQuad3, PhasefieldNonLin>;
