/** Nichtlinearer Finite-Elemente-Operator fuer das Programmierpraktikum
 *
 *  \file   NonlinearFEOperator.h
 *  \author Stefan W. von Deylen, Benedict Geihe
 *  \date   21.11.2008
 *
 */

#ifndef __NONLINEARFEOPERATOR_H
#define __NONLINEARFEOPERATOR_H

#include "quadrature.h"
#include "../vecmat/vector.h"


//! Klasse fuer nichtlinearen Finite-Elemente-Operator
//! Template-Argumente fuer Wertebereich, Quadraturregel und Integranden (delete flag pointer concept)
template <typename RangeType, typename QuadType, typename IntegrandType>
class NonlinearFEOperator
{
    public:
        explicit NonlinearFEOperator( const Grid & grid );
        NonlinearFEOperator( const Grid & grid, const IntegrandType & integrand );

        ~NonlinearFEOperator();

        void applyAdd( const Vector& x, RangeType& b, double lambda = 1.0 ) const;

    protected:
        const Grid & _grid;
        const IntegrandType * _integrand;
        bool _deleteIntegrandOnDestruction;
};

#endif
