/** Phasefield related operators
 *
 *  \file   PhasefieldOps.cpp
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   07.12.2012
 *
 */


#include "PhasefieldOps.h"

/*************************************************************************************************/
/*  Class PhasefieldNonLin                                                                       */
/*************************************************************************************************/
//! evaluate \f$ (v^2-1)^2 \; w_q \f$
void PhasefieldNonLin::addEval( const Vector& nodalValues, double& b, const Triangle & t, const BarCoord& q, const double w ) const
{
    // Interpolate linear phasefield at quadrature point
    double ret = t.calcFunctionValue( nodalValues, q );
    ret *= ret;
    ret -= 1.;

    // return value has to be multiplied by w and then added to b
    b += ret * ret * w;
}
// ------------------------------------------------------------------------------------------------


/*************************************************************************************************/
/*  Class PhasefieldElasticEnergy                                                                */
/*************************************************************************************************/
//! constructor
PhasefieldElasticEnergy::PhasefieldElasticEnergy( ElasticProblem & ep, double eps, double eta, double nu )
: _ep        ( ep )
, _mBilf     ()
, _massOp    ( ep.getGrid(), _mBilf )
, _sBilf     ()
, _stiffOp   ( ep.getGrid(), _sBilf )
, _eps       ( eps )
, _eta       ( eta )
, _nu        ( nu )
{ }

//! applyAdd-Methode
void PhasefieldElasticEnergy::applyAdd( const Vector& pf, double& energy, double lambda ) const
{
    //! @todo compute volume term
    double vol = 0.;

    //! @todo compute length term
    double length = 0.;

    //! @todo compute elastic energy
    double elasticEnergy = 0.;

    // the current grid is available as _ep.getGrid()
    // the current elastic solution is available as _ep.getSolution()

    energy += lambda * ( .5 * elasticEnergy + _nu * vol +  _eta * length );
}
// ------------------------------------------------------------------------------------------------


/*************************************************************************************************/
/*  Class PhasefieldElasticEnergyGrad                                                            */
/*************************************************************************************************/
//! constructor
PhasefieldElasticEnergyGrad::PhasefieldElasticEnergyGrad( ElasticProblem & ep,
                                                          double eps, double eta, double nu )
: _ep        ( ep )
, _mBilf     ()
, _massOp    ( ep.getGrid(), _mBilf )
, _sBilf     ()
, _stiffOp   ( ep.getGrid(), _sBilf )
, _eps       ( eps )
, _eta       ( eta )
, _nu        ( nu )
{ }

//! applyAdd-Methode
void PhasefieldElasticEnergyGrad::applyAdd( const Vector& pf, Vector& grad, double lambda ) const
{
    //! @todo compute volume term gradient

    //! @todo compute length term gradient

    //! @todo compute gradient of elastic energy w.r.t. phasefield
}
// ------------------------------------------------------------------------------------------------
