/** Phasefield related operators
 *
 *  \file   PhasefieldOps.h
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   07.12.2012
 *
 */


#ifndef __PHASEFIELDOPS_H
#define __PHASEFIELDOPS_H

#include "../triang/triangle.h"
#include "../vecmat/operator.h"
#include "bilf.h"
#include "elastBilf.h"
#include "LinearFEOperator.h"
#include "NonlinearFEOperator.h"
#include "ElasticProblem.h"


//! integrand class for nonlinear part in lenght term
class PhasefieldNonLin
{
    public:
        void addEval( const Vector& nodalValues, double& b, const Triangle & t, const BarCoord& q, const double w ) const;
};

//! class for evaluation of elastic energy in shape optimization
class PhasefieldElasticEnergy : public Operator<Vector, double>
{
    public:
        PhasefieldElasticEnergy( ElasticProblem & ep, double eps, double eta, double nu );
        void applyAdd( const Vector& pf, double& energy, double lambda = 1.0 ) const;

    protected:
        ElasticProblem & _ep;
        MassBilf _mBilf;
        const LinearFEOperator< MassBilf,  BarQuad2 > _massOp;
        StiffBilf _sBilf;
        const LinearFEOperator< StiffBilf, BarQuad1 > _stiffOp;
        const double _eps;
        const double _eta;
        const double _nu;
};

//! class for gradient evaluation of elastic energy in shape optimization
class PhasefieldElasticEnergyGrad : public Operator<Vector, Vector>
{
    public:
        PhasefieldElasticEnergyGrad( ElasticProblem & ep, double eps, double eta, double nu );
        void applyAdd( const Vector& pf, Vector& grad, double lambda = 1.0 ) const;

    protected:
        ElasticProblem & _ep;
        MassBilf _mBilf;
        const LinearFEOperator< MassBilf,  BarQuad2 > _massOp;
        StiffBilf _sBilf;
        const LinearFEOperator< StiffBilf, BarQuad1 > _stiffOp;
        const double _eps;
        const double _eta;
        const double _nu;
};

#endif
