/** Bilinearformen
 *
 *  \file   bilf.cpp
 *  \author Benedict Geihe
 *  \date   19.04.2009
 *
 */

#include "bilf.h"

/***************************************************************************************************/
/*                                    Bilinear Functions                                           */
/***************************************************************************************************/
//! Wertet \f$ \varphi_i \varphi_j \f$ auf einem Dreieck aus
double MassBilf::operator() (const Triangle & /*t*/, const BarCoord & q, int i, int j ) const
{
    return q[i] * q[j];
}

//! Wertet \f$ \nabla \varphi_i \nabla \varphi_j \f$ auf einem Dreieck aus
double StiffBilf::operator() (const Triangle & t, const BarCoord & /*q*/, int i, int j ) const
{
    if (i > j)
        swap(i, j);

    // kartesische Gradienten sind AmTrans * e_i mit
    // eukl. Einheitsvektoren e_i:
    SmallVector2<> Ae0 ( t._AmTrans(0,0), t._AmTrans(1,0) ),
                   Ae1 ( t._AmTrans(0,1), t._AmTrans(1,1) ),
                   Ae2 ( t._AmTrans(0,2), t._AmTrans(1,2) );

    if (i == 0)
    {
        if (j == 0)
            return Ae0 * Ae0;
        else if (j == 1)
            return Ae0 * Ae1;
        else
            return Ae0 * Ae2;
    }
    if (i == 1)  {
        if (j == 1)
            return Ae1 * Ae1;
        else
            return Ae1 * Ae2;
    }
    return Ae2 * Ae2;
}
