/** Boundary masks fuer das Programmierpraktikum
 *
 *  \file   boundaryMasks.h
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   15.10.2012
 *
 */

#ifndef __BOUNDARYMASKS_H
#define __BOUNDARYMASKS_H

class PeriodicBoundaryMask;

#include "../triang/grid.h"
#include "../triang/smallVector.h"
#include "../vecmat/sparse.h"


//! \brief Basisklasse für Randdaten Maskierung
class BoundaryMask
{
    public:
        virtual void apply ( SparseMatrix & matrix ) const = 0;
};


//! Mask Klasse fuer Dirichlet Randwerte
class DirichletBoundaryMask : public BoundaryMask
{
    public:
        DirichletBoundaryMask();
        DirichletBoundaryMask( const Grid& grid );

        void mask ( const int index );
        void apply ( SparseMatrix & matrix ) const;
        void apply ( Vector & vec ) const;

    protected:
        vector<int> _mask;
};


//! Mask Klasse fuer periodische Randwerte
class PeriodicBoundaryMask : public vector< SmallVector2<int> >
{
    public:
        PeriodicBoundaryMask( const Grid& grid );

        void apply( SparseMatrix & matrix ) const;

        void collapse( Vector & vec ) const;
        void extend( Vector & vec ) const;
    protected:
        const Grid & _grid;
};


#endif
