/** Bilinear form for two scale problems
 *
 *  \file   twoscaleBilf.h
 *  \author Benedict Geihe, Behrend Heeren, Sascha Tölkes
 *  \date   24.10.2012
 *
 */

#ifndef __TWOSCALEBILF_H
#define __TWOSCALEBILF_H

#include "LinearFEOperator.h"
#include "diffusiveBilf.h"
#include "quadrature.h"
#include "../vecmat/solver.h"
#include "boundaryMasks.h"


//! This structure shall carry every information needed to solve a cell problem: grids, phasefields, solutions, ...
template <typename MicroBilfType>
class CellProblem
{
    public:
        CellProblem ( const MicroBilfType & bilf );
              MicroBilfType & getBilfReference ();
        const MicroBilfType & getBilfReference () const;

        void recomputeBaseSolutions();
        double evaluateMicroEnergy( const SmallVector2<> & grad1, const SmallVector2<> & grad2 ) const;

        Vector& getBaseSolutionReference(unsigned int b);

    protected:
        // helper
        void generateLinearProfile( const SmallVector2<> & grad, Vector & linearProfile ) const;

        // members
        Grid          _grid;
        MicroBilfType _bilf;
        SparseMatrix  _stiffMat;
        Vector        _baseSolution1;
        Vector        _baseSolution2;
};



//! Bilinear form for accessing microscopic cell problems
template <typename MicroBilfType>
class TwoscaleBilf
{
    public:
        TwoscaleBilf( const Grid & macroGrid, const MicroBilfType & microBilfRepresentative );
              MicroBilfType & getBilfReference ( unsigned int index );
        const MicroBilfType & getBilfReference ( unsigned int index ) const;
        void update();

        double operator() (const Triangle & t, const BarCoord & q, int i, int j) const;

    protected:
        const Grid &                        _macroGrid;
        vector<CellProblem<MicroBilfType> > _cellProblems;
        vector<bool>                        _cellProblemChanged;
};

#endif
