void generateMysteriousGrid( Grid & grid )
{
    vector<Node> nodes;
    nodes.push_back(Node(-1., 0.));
    nodes.push_back(Node( 1., 0.));
    nodes.push_back(Node( 0., 1.));
    nodes.push_back(Node(-3., 2.));
    nodes.push_back(Node(-1., 2.));
    nodes.push_back(Node( 1., 2.));
    nodes.push_back(Node( 3., 2.));
    nodes.push_back(Node(-2., 4.));
    nodes.push_back(Node(-1., 4.));
    nodes.push_back(Node( 0., 4.));
    nodes.push_back(Node( 1., 4.));
    nodes.push_back(Node( 2., 4.));
    nodes.push_back(Node(-1.25, 5.5));
    nodes.push_back(Node(-0.5, 5.5));
    nodes.push_back(Node( 0.5, 5.5));
    nodes.push_back(Node( 1.25, 5.5));
    nodes.push_back(Node( 0., 6.75));

    grid = Grid(nodes[0], nodes[1], nodes[2]);
    grid.addNodeAndTriangle ( nodes[ 4], Grid::DartIterator ( grid.getTriangle( 0), 1, 0 ) );
    grid.addNodeAndTriangle ( nodes[ 5], Grid::DartIterator ( grid.getTriangle( 1), 1, 2 ) );
    grid.addTriangle ( Grid::DartIterator ( grid.getTriangle(2), 2 ), Grid::DartIterator ( grid.getTriangle(0), 0 ) );
    grid.addNodeAndTriangle ( nodes[ 9], Grid::DartIterator ( grid.getTriangle( 2), 1, 0 ) );
    grid.addNodeAndTriangle ( nodes[ 8], Grid::DartIterator ( grid.getTriangle( 4), 1, 0 ) );
    grid.addNodeAndTriangle ( nodes[ 3], Grid::DartIterator ( grid.getTriangle( 5), 1, 0 ) );
    grid.addNodeAndTriangle ( nodes[10], Grid::DartIterator ( grid.getTriangle( 4), 2, 0 ) );
    grid.addNodeAndTriangle ( nodes[ 6], Grid::DartIterator ( grid.getTriangle( 7), 1, 0 ) );
    grid.addNodeAndTriangle ( nodes[13], Grid::DartIterator ( grid.getTriangle( 5), 2, 0 ) );
    grid.addNodeAndTriangle ( nodes[ 7], Grid::DartIterator ( grid.getTriangle( 9), 2, 0 ) );
    grid.addNodeAndTriangle ( nodes[14], Grid::DartIterator ( grid.getTriangle( 7), 2, 0 ) );
    grid.addNodeAndTriangle ( nodes[11], Grid::DartIterator ( grid.getTriangle(11), 2, 0 ) );
    grid.addTriangle ( Grid::DartIterator ( grid.getTriangle(9), 1 ), Grid::DartIterator ( grid.getTriangle(11), 1 ) );
    grid.addNodeAndTriangle ( nodes[16], Grid::DartIterator ( grid.getTriangle(13), 0, 1 ) );
    grid.addNodeAndTriangle ( nodes[12], Grid::DartIterator ( grid.getTriangle(14), 2, 0 ) );
    grid.addNodeAndTriangle ( nodes[15], Grid::DartIterator ( grid.getTriangle(14), 1, 2 ) );

    bool done = false;
    do
    {
        done = true;
        for (Grid::TriangleIterator tIter = grid; tIter.notAtEnd(); ++tIter)
        {
            if ( tIter->getLongestEdgeLength() > .2 )
            {
                tIter->markForRefinement();
                done = false;
            }
        }
        grid.refineMarkedTriangles();
    } while (!done);
}

void generateMysteriousPhasefield( const Grid & grid, Vector & pf )
{
    const double eps = 2e-1;
    SmallVector2<> diff;
    double dist;
    vector< SmallVector2<> > mids;
    vector< double > radius;
    mids.push_back( SmallVector2<>(-2.  ,2.4) );
    radius.push_back(.3);
    mids.push_back( SmallVector2<>( 2.  ,2.4) );
    radius.push_back(.3);
    mids.push_back( SmallVector2<>(- .9 ,3.1) );
    radius.push_back(.25);
    mids.push_back( SmallVector2<>(  .9 ,3.1) );
    radius.push_back(.25);
    mids.push_back( SmallVector2<>(-1.25,4.3) );
    radius.push_back(.2);
    mids.push_back( SmallVector2<>( 1.25,4.3) );
    radius.push_back(.2);
    mids.push_back( SmallVector2<>(- .8 ,5.7) );
    radius.push_back(.1);
    mids.push_back( SmallVector2<>(  .8 ,5.7) );
    radius.push_back(.1);
    mids.push_back( SmallVector2<>( 0.  ,6.2) );
    radius.push_back(.2);

    for (Grid::NodeConstIterator nIter = grid; nIter.notAtEnd(); ++nIter)
    {
        if ( (*nIter)[1] < 5. )
            dist = fabs( (*nIter)[0] );
        else  {
            diff[0] = 0.; diff[1] = 5.;
            diff -= (*nIter);
            dist = diff.norm_l2();
        }
        dist -= .2;
        for ( unsigned int i = 0; i < mids.size(); ++i )
        {
            diff = mids[i];
            diff -= (*nIter);
            dist = std::min( dist, diff.norm_l2()-radius[i] );
        }
        pf[ nIter.getIndex() ] = tanh ( -3./4. * dist / eps );
    }
}
