/** Lab exercise 1
 *
 *  \file   problem01.cpp
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   12.10.2012
 *
 *  Löse
 *  
 *                        -div( a(x) \nabla u ) = f in Omega
 *                                            u = 0 auf dem Rand
 * 
 *  mit a(x) = 1 und Omega = [0,1]^2.
 *
 * --------------------------------------------------------------------------------------------- */


#include <iostream>
#include <stdlib.h>

using namespace std;

#include "fe/LinearFEOperator.h"
#include "fe/bilf.h"
#include "fe/boundaryMasks.h"
#include "vecmat/solver.h"

double linearProfile (const Node & x)
{
    SmallVector2<> n (1., 2.);
    const double m  = 1.;

    return m / n.norm_l2q() * ( n[1]*x[0] - n[0]*x[1] );
}


int main()
{
    // erstelle Gitter mit Gittertiefe 'gridDepth'
    int gridDepth = 8;
    Grid grid = Grid::createRectangularGrid( 0., 0., 1., 1., gridDepth );

    // initialisiere Randmaske
    DirichletBoundaryMask bMask ( grid );

    // initialisiere Variablen
    Vector u(grid),
           rhs(grid),
           f(grid);

    // setze  Randwerte
    for (Grid::NodeConstIterator nIter = grid; nIter.notAtEnd(); ++nIter)
      f[ nIter.getIndex() ] = linearProfile(*nIter);

    // plotte Quellterm
    char sol_filename[12];
    sprintf(sol_filename, "f");
    grid.plotSolution(f, sol_filename, JPG, true);

    // erstelle FE-Operatoren, behandle rechte Seite
    StiffBilf sBilf;
    LinearFEOperator< StiffBilf, BarQuad1 > stiffOp(grid,sBilf);
    
    MassBilf mBilf;
    LinearFEOperator< MassBilf, BarQuad2 > massOp(grid,mBilf);
    massOp.apply( f, rhs );
    
    // setze Randadaten auf 0
    bMask.apply( stiffOp );
    bMask.apply( rhs );   
    
    // plotte rechte Seite
    sprintf(sol_filename, "rhs");
    grid.plotSolution(rhs, sol_filename, JPG, true);
    
    // loese lin. GlS
    Solver<SparseMatrix> solver(stiffOp, true);
    solver.setMaxIter (1000);
    solver.setTol(1E-8);
    solver.CGapply(rhs, u);
    
    // plotte Lösung
    sprintf(sol_filename, "solution");
    grid.plotSolution(u, sol_filename, JPG, true);
    
#if defined(__MINGW32_VERSION) || defined(_MSC_VER) || defined(__BCPLUSPLUS__)
    system("PAUSE");
#endif
    return 0;
}
