/** Lab exerice 10
 *
 *  \file   problem10.cpp
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   22.01.2013
 *
 *  Evaluate the discrete energy J[V,U] and gradient J_{,V}[V,U] in the corresponding
 *  applyAdd methods of \a PhasefieldElasticEnergy and \a PhasefieldElasticEnergyGrad
 *  in fe/PhasefieldOps.cpp
 *
 * --------------------------------------------------------------------------------------------- */



#include <cstdlib>
#include <iostream>

using namespace std;

#include "fe/ElasticProblem.h"
#include "fe/PhasefieldOps.h"
#include "algorithms/gradientDescent.h"
#include "algorithms/derivativeChecker.h"


int main()
{
  try  {
    const ScenarioType scenario = SQUARE;
    char filename[20];
    double eps = 2e-2;
    double eta = 5e-3;
    double nu  = 2e-1;

    // setup elastic problem
    ElasticProblem elast( eps, scenario );
    // reference to current grid used for plotting
    const Grid & grid = elast.getGrid();
    // make a copy
    Vector pf = elast.getPhasefield();
    grid.plotSolutionDeformed( pf, elast.getSolution(), "initial", JPG );

    // construct operators
    PhasefieldElasticEnergy     energyOp   ( elast, eps, eta, nu );
    PhasefieldElasticEnergyGrad gradientOp ( elast, eps, eta, nu );

    // derivative check
//     DerivativeChecker dc ( energyOp, gradientOp );
//     dc.check(4,pf);
//     dc.checkAll(pf);

    // set up gradient descent
    GradientDescent gradientDescent( energyOp, gradientOp );
    gradientDescent.init();
    int iter = 0;
    while ( gradientDescent.performSingleStep(pf) == NotTerminated )
    {
        ++iter;
        sprintf(filename, "pf_%03i", iter );
//         grid.plotSolutionDeformed( pf, elast.getSolution(), filename, JPG );
        grid.plotSolution( pf, filename, JPG, true, -1.1, 1.1 );
    };
    gradientDescent.checkTermination();
    grid.plotSolutionDeformed( pf, elast.getSolution(), "final", JPG );

  } catch ( string err ) { cerr << err << endl; }

  return 0;
}
