/** Klasse fuer kleine Vektoren (Koordinaten etc.)
 *
 *  \file   smallVector.h
 *  \author Stefan W. von Deylen, Benedict Geihe
 *  \date   18.09.2008
 *
 */

#ifndef __smallVector_h
#define __smallVector_h

#include <iostream>
#include <iomanip>
#include <cassert>
#include <cmath>

using namespace std;

//! Klasse fuer kleine Vektoren
template < unsigned int Size, typename DataType = double >
class SmallVector
{
    public:
        // Standard-Konstruktor
        // Hinweis: Kopierkonstruktor, Destruktor und Zuweisungsoperator werden automatisch sinnvoll implementiert
        SmallVector();

        // Funktionen
        void setZero();
        void normalize();

        // Normen
        double   norm_l2() const;
        DataType norm_l2q() const;
        DataType norm_linfty() const;

        // Zugriff
        const DataType & operator[] ( unsigned int i ) const;
              DataType & operator[] ( unsigned int i );

        // Rechnen
        void setToMultiple( const DataType d, const SmallVector< Size, DataType > & v );
        void addMultiple  ( const DataType d, const SmallVector< Size, DataType > & v );
        SmallVector< Size, DataType > & operator*=( const DataType r );
        SmallVector< Size, DataType > & operator/=( const DataType r );
        SmallVector< Size, DataType > & operator+=( const SmallVector< Size, DataType > & v );
        SmallVector< Size, DataType > & operator-=( const SmallVector< Size, DataType > & v );
        DataType operator*( const SmallVector< Size, DataType > & v ) const;

        // Vergleiche
        bool operator==(const SmallVector<Size, DataType> & _alius) const;
        bool operator!=(const SmallVector<Size, DataType> & _alius) const;

    private:
        DataType _entry[ Size ];
};

//! Ausgabefunktion mit <<-Operator
template < unsigned int Size, typename DataType >
ostream& operator<< ( ostream & stream, const SmallVector< Size, DataType > & v )
{
    stream.precision(2);
    stream.setf( ios::fixed, ios::floatfield );

    stream << "[";
    for ( unsigned int i = 0; i < Size; i++ )
    {
        stream << " " << setw(8) << v[i] << " ";
    }
    stream << "]" << endl;

    stream.precision(6);
    stream.unsetf( ios::fixed );

    return stream;
}


// ------------------------------------------------------------------------------------------------


//! Klasse explizit fuer Vektoren der Laenge 2
template < typename DataType = double >
class SmallVector2 : public SmallVector< 2, DataType >
{
    public:
        SmallVector2();
        SmallVector2( DataType v1, DataType v2 );
        SmallVector2( const SmallVector<2,DataType> & v );
};


// ------------------------------------------------------------------------------------------------


//! Klasse explizit fuer Vektoren der Laenge 3
template < typename DataType = double >
class SmallVector3 : public SmallVector< 3, DataType >
{
    public:
        SmallVector3();
        SmallVector3( DataType v1, DataType v2, DataType v3 );
        SmallVector3( const SmallVector<3, DataType> & v );
};


// ------------------------------------------------------------------------------------------------


//! Absolutbetrag, funktioniert fuer int und double
template < typename DataType >
DataType Abs( DataType x )
{
    return (x >= 0 ? x : -x);
}

#endif
