/** Matrix-Klasse für das Programmierpraktikum
 *
 *  \file   matrix.h
 *  \author Stefan W. von Deylen, Benedict Geihe
 *  \date   18.09.2008
 *
 */

#ifndef __MATRIX_H
#define __MATRIX_H

#include "vector.h"
#include "../triang/smallVector.h"


//! \brief Matrix-Klasse
//!
//! _n zeilen, _m spalten, beginnend mit 0
//! gespeichert als Vektor: Element (i,j) steht an Stelle ( i*_m + j )
class Matrix : protected Vector
{
    public:
        // Konstruktoren, Destruktor, Zuweisungsoperator
        Matrix();
        Matrix( unsigned int n, unsigned int m );
        Matrix( const Matrix& mat );
        ~Matrix();
        Matrix& operator=( const Matrix& mat );

        // Übernahme von Vector-Funktionalität
        using Vector::setZero;
        using Vector::setToMultiple;
        using Vector::addMultiple;

        // Dimensionsausgabe
        unsigned int getRows() const;
        unsigned int getCols() const;

        // Zugriff
        double  operator()( unsigned int i, unsigned int j ) const;
        double& operator()( unsigned int i, unsigned int j );

        // Funktionen
        void apply   ( const Vector& x, Vector& b, double lambda = 1.0 ) const;
        void applyAdd( const Vector& x, Vector& b, double lambda = 1.0 ) const;

        template <unsigned int n, unsigned int m>
        void apply( const SmallVector<m>& x, SmallVector<n>& b, double lambda = 1.0 ) const;
        template <unsigned int n, unsigned int m>
        void applyAdd( const SmallVector<m>& x, SmallVector<n>& b, double lambda = 1.0 ) const;

        void mmult( const Matrix& A, const Matrix& B);

        void transpose();

        // Normen
        double norm_l1() const;
        double norm_linf() const;

    private:
        unsigned int _n;
        unsigned int _m;
};

//! Ausgabefunktion mit <<-Operator
ostream& operator<<(ostream& stream, const Matrix& matrix);

//! Spezielle apply-Funktion fuer SmallVector
template <unsigned int n, unsigned int m>
void Matrix::apply( const SmallVector<m>& x, SmallVector<n>& b, double lambda ) const
{
    b.setZero();
    for ( unsigned int i = 0; i < _n; i++ )
    {
        for ( unsigned int j = 0; j < _m; j++ )
        {
            b[i] += (*this)(i,j) * x[j];
        }
        b[i] *= lambda;
    }
}

//! Spezielle applyAdd-Funktion fuer SmallVector
template <unsigned int n, unsigned int m>
void Matrix::applyAdd( const SmallVector<m>& x, SmallVector<n>& b, double lambda ) const
{
    b.setZero();
    for ( unsigned int i = 0; i < _n; i++ )
    {
        for ( unsigned int j = 0; j < _m; j++ )
        {
            b[i] += (*this)(i,j) * x[j] * lambda;
        }
    }
}

#endif
