#ifndef __solver_h
#define __solver_h

#include "vector.h"

using namespace std;

template <typename MatrixType>
class Solver
{
    public:
        Solver( const MatrixType & matrix, bool verbose = false );

        void CGapply   ( const Vector & b, Vector & x );
        void CGapplyAdd( const Vector & b, Vector & x );

        void projectingCGapply    ( const Vector & constr, const Vector & b, Vector & x );
        void projectingCGapplyAdd ( const Vector & constr, const Vector & b, Vector & x );

        void setVerbose( bool verbose = true );
        void setTol( double tol );
        void setMaxIter( int maxiter );
        void setProjectionThreshold( double threshold );

    protected:
        const MatrixType & _matrix;

    private:
        void printHeader() const;
        void printIterate() const;

        bool   _verbose;
        double _tol, _error, _firsterror, _lasterror, _constrViolation, _projectionThreshold;
        int    _maxiter, _iter;
        Vector* _constr;
};

#endif
