#ifndef __sparse_H
#define __sparse_H

#include <iostream>
#include <iomanip>
#include <cassert>
#include <vector>       // STL-Vektor
#include <map>          // STL-Map

#include "vector.h"     // "unser" Vektor

typedef map<int, double> SparseRow;

using namespace std;

//! SparseMatrix-Klasse: n zeilen, m spalten, beginnend mit 0
//! gespeichert als Vektor (Laenge n) von (int->double) Maps
class SparseMatrix : protected vector<SparseRow>
{
    public:
        // Konstruktoren, Destruktor
        SparseMatrix();
        SparseMatrix( unsigned int n, unsigned int m );
        SparseMatrix( const SparseMatrix& mat );
        ~SparseMatrix();

        // Dimensionsausgabe
        unsigned int getRows() const;
        unsigned int getCols() const;

        // Zugriff
        double  operator()( unsigned int i, unsigned int j ) const;
        double& operator()( unsigned int i, unsigned int j );

        // Funktionen
        void setZero();
        void setZero( unsigned int i, unsigned int j );
        void setToMultiple( const double d, const SparseMatrix& mat );
        void addMultiple  ( const double d, const SparseMatrix& mat );
        void apply( const Vector& x, Vector& b, double lambda = 1.0 ) const;
        void applyAdd( const Vector& x, Vector& b, double lambda = 1.0 ) const;
        void mmult( const SparseMatrix& A, const SparseMatrix& B);

        // Zeilen- / Spaltenoperationen
        void setRowColToDiagonal ( const int index, double diagEntry = 1. );
        void addMultipleCol( const double d, const int fromCol, const int toCol );
        void addMultipleRow( const double d, const int fromCol, const int toCol );
        
        // Normen
        double norm_l1() const;
        double norm_linf() const;

    private:
        unsigned int _n;
        unsigned int _m;
};

//! Ausgabefunktion mit <<-Operator
ostream& operator<<(ostream& stream, const SparseMatrix& matrix);

#endif
