/** Tensor-Klasse für das Programmierpraktikum
 *
 *  \file   tensor.cpp
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   12.10.2012
 *
 */


#include "tensor.h"


BlobTensor::BlobTensor( const SmallVector2<> & midpoint, const double epsilon, const double inflection,
                        const double lowerBound, const double upperBound )
    : _midpoint( midpoint )
    , _epsilon( epsilon )
    , _inflection( inflection )
    , _lowerBound( lowerBound )
    , _upperBound( upperBound )
{}

BlobTensor * BlobTensor::clone () const
{
    return new BlobTensor( *this );
}

void BlobTensor::setMidpoint( const SmallVector2<> & midpoint )  {
    _midpoint = midpoint;
}

void BlobTensor::setEpsilon( const double epsilon )  {
    _epsilon = epsilon;
}

void BlobTensor::setInflection( const double inflection )  {
    _inflection = inflection;
}

void BlobTensor::setLowerBound( const double lowerBound )  {
    _lowerBound = lowerBound;
}

void BlobTensor::setUpperBound( const double upperBound )  {
    _upperBound = upperBound;
}


double BlobTensor::evaluate ( const Triangle & t, const BarCoord & q ) const
{
    SmallVector2<> cartCoord;
    t.calcCartesianCoord ( q, cartCoord );
    return evaluate( cartCoord );
}

double BlobTensor::evaluate( const SmallVector2<> & cartCoord ) const {
    SmallVector2<> diff(cartCoord);
    diff.addMultiple( -1., _midpoint );
    double dist = diff.norm_l2();
    return (_upperBound-_lowerBound) * (.5 - M_1_PI * atan( (dist-_inflection)/_epsilon )) + _lowerBound;  
}

void BlobTensor::evaluateGrad ( const Triangle & t, const BarCoord & q, SmallVector2<>& grad ) const
{
    t.calcCartesianCoord ( q, grad );
    grad.addMultiple( -1., _midpoint );
    double dist = grad.norm_l2();
    grad *=  (_lowerBound-_upperBound) * M_1_PI * derivativeOfAtan( (dist - _inflection) / _epsilon ) / (dist*_epsilon) ;
}

double BlobTensor::derivativeOfAtan( const double x ) const {
  return 1.0/(1+x*x);  
}
