/** Tensor-Klasse für das Programmierpraktikum
 *
 *  \file   tensor.h
 *  \author Benedict Geihe, Behrend Heeren
 *  \date   12.10.2012
 *
 */

#ifndef __TENSOR_H
#define __TENSOR_H

#include "../triang/triangle.h"


//! \brief Tensor-Klasse
class TensorOrder0
{
    public:
        virtual TensorOrder0 * clone () const = 0;
        virtual double evaluate ( const Triangle & t, const BarCoord & q ) const = 0;
        virtual void evaluateGrad ( const Triangle & t, const BarCoord & q, SmallVector2<>& grad ) const = 0;
};

//! \brief Tensor-Klasse
class BlobTensor : public TensorOrder0
{
    public:
        BlobTensor( const SmallVector2<> & midpoint = SmallVector2<>(.5,.5),
                    const double epsilon = 0.025,
                    const double inflection = .5,
                    const double lowerBound = 0.,
                    const double upperBound = 1.
                  );

        // "virtual copy constructor"
        BlobTensor * clone () const;

        void setMidpoint( const SmallVector2<> & midpoint );
        void setEpsilon( const double epsilon );
        void setInflection( const double inflection );
        void setLowerBound( const double lowerBound );
        void setUpperBound( const double upperBound );

        double evaluate( const SmallVector2<> & cartCoord ) const;
        double evaluate ( const Triangle & t, const BarCoord & q ) const;
        void evaluateGrad ( const Triangle & t, const BarCoord & q, SmallVector2<>& grad ) const;

    protected:
        double derivativeOfAtan( const double x ) const;
      
        SmallVector2<> _midpoint;
        double _epsilon;
        double _inflection;
        double _lowerBound;
        double _upperBound;
};

#endif
