/** Vektor-Klasse für das Programmierpraktikum
 *
 *  \file   vector.h
 *  \author Stefan W. von Deylen, Benedict Geihe
 *  \date   16.09.2008
 *
 */

#ifndef __vector_H
#define __vector_H

#include <iostream>
#include <iomanip>
#include <cmath>
#include <cassert>
#include <vector>  // STL-Vektor

class Grid;

using namespace std;

//! \brief Vektor-Klasse
//!
//! abgeleitet vom STL-Vektor (Typ: double)
class Vector : public vector<double>
{
    public:
        // Konstruktoren, Destruktor, Zuweisungsoperator
        Vector();
        explicit Vector( unsigned int n );
        explicit Vector( const Grid & g );
        Vector( const Vector& v );
        ~Vector();
        Vector& operator=( const Vector& v);
	void resize( unsigned int n );

        // Funktionen
        void setAll( double value );
        void setZero();
        void normalize();

        // Normen
        double norm_l2() const;
        double norm_l2q() const;
        double norm_linf() const;

        // Rechnen
        void setToMultiple( const double d, const Vector& v );
        void addMultiple  ( const double d, const Vector& v );
        void operator*=( const double r );
        void operator/=( const double r );
        void operator+=( const Vector& v );
        void operator-=( const Vector& v );
        // Skalarprodukt
        double operator*( const Vector& v ) const;

        // Vergleiche
        bool operator==( const Vector & alius ) const;
        bool operator!=( const Vector & alius ) const;
};

//! Ausgabefunktion mit <<-Operator
ostream& operator<<(ostream&, const Vector&);

#endif
